@@ -123,6 +123,15 @@ class DLLSYM CLIST
       int comparator (           //comparison routine
       const void *, const void *));
 
+    // Assuming list has been sorted already, insert new_data to
+    // keep the list sorted according to the same comparison function.
+    // Comparision function is the same as used by sort, i.e. uses double
+    // indirection. Time is O(1) to add to beginning or end.
+    // Time is linear to add pre-sorted items to an empty list.
+    // If unique, then don't add duplicate entries.
+    void add_sorted(int comparator(const void*, const void*),
+                    bool unique, void* new_data);
+
     void internal_dump (         //serialise each elem
       FILE * f,                  //to this file
       void element_serialiser (  //using this function
@@ -198,7 +207,7 @@ class DLLSYM CLIST_ITERATOR
                          CLIST *list_to_add);  //move to it 1st item
 
     void *data() {  //get current data
-    #ifdef _DEBUG
+    #ifndef NDEBUG
       if (!list)
         NO_LIST.error ("CLIST_ITERATOR::data", ABORT, NULL);
       if (!current)
@@ -221,7 +230,7 @@ class DLLSYM CLIST_ITERATOR
     void mark_cycle_pt();  //remember current
 
     BOOL8 empty() {  //is list empty?
-    #ifdef _DEBUG
+    #ifndef NDEBUG
       if (!list)
         NO_LIST.error ("CLIST_ITERATOR::empty", ABORT, NULL);
     #endif
@@ -261,7 +270,7 @@ class DLLSYM CLIST_ITERATOR
 
 inline void CLIST_ITERATOR::set_to_list(  //change list
                                         CLIST *list_to_iterate) {
-  #ifdef _DEBUG
+  #ifndef NDEBUG
   if (!this)
     NULL_OBJECT.error ("CLIST_ITERATOR::set_to_list", ABORT, NULL);
   if (!list_to_iterate)
@@ -302,7 +311,7 @@ inline void CLIST_ITERATOR::add_after_then_move(  // element to add
                                                 void *new_data) {
   CLIST_LINK *new_element;
 
-  #ifdef _DEBUG
+  #ifndef NDEBUG
   if (!this)
     NULL_OBJECT.error ("CLIST_ITERATOR::add_after_then_move", ABORT, NULL);
   if (!list)
@@ -352,7 +361,7 @@ inline void CLIST_ITERATOR::add_after_stay_put(  // element to add
                                                void *new_data) {
   CLIST_LINK *new_element;
 
-  #ifdef _DEBUG
+  #ifndef NDEBUG
   if (!this)
     NULL_OBJECT.error ("CLIST_ITERATOR::add_after_stay_put", ABORT, NULL);
   if (!list)
@@ -405,7 +414,7 @@ inline void CLIST_ITERATOR::add_before_then_move(  // element to add
                                                  void *new_data) {
   CLIST_LINK *new_element;
 
-  #ifdef _DEBUG
+  #ifndef NDEBUG
   if (!this)
     NULL_OBJECT.error ("CLIST_ITERATOR::add_before_then_move", ABORT, NULL);
   if (!list)
@@ -452,7 +461,7 @@ inline void CLIST_ITERATOR::add_before_stay_put(  // element to add
                                                 void *new_data) {
   CLIST_LINK *new_element;
 
-  #ifdef _DEBUG
+  #ifndef NDEBUG
   if (!this)
     NULL_OBJECT.error ("CLIST_ITERATOR::add_before_stay_put", ABORT, NULL);
   if (!list)
@@ -497,7 +506,7 @@ inline void CLIST_ITERATOR::add_before_stay_put(  // element to add
  **********************************************************************/
 
 inline void CLIST_ITERATOR::add_list_after(CLIST *list_to_add) {
-  #ifdef _DEBUG
+  #ifndef NDEBUG
   if (!this)
     NULL_OBJECT.error ("CLIST_ITERATOR::add_list_after", ABORT, NULL);
   if (!list)
@@ -547,7 +556,7 @@ inline void CLIST_ITERATOR::add_list_after(CLIST *list_to_add) {
  **********************************************************************/
 
 inline void CLIST_ITERATOR::add_list_before(CLIST *list_to_add) {
-  #ifdef _DEBUG
+  #ifndef NDEBUG
   if (!this)
     NULL_OBJECT.error ("CLIST_ITERATOR::add_list_before", ABORT, NULL);
   if (!list)
@@ -597,7 +606,7 @@ inline void CLIST_ITERATOR::add_list_before(CLIST *list_to_add) {
 inline void *CLIST_ITERATOR::extract() {
   void *extracted_data;
 
-  #ifdef _DEBUG
+  #ifndef NDEBUG
   if (!this)
     NULL_OBJECT.error ("CLIST_ITERATOR::extract", ABORT, NULL);
   if (!list)
@@ -608,23 +617,21 @@ inline void *CLIST_ITERATOR::extract() {
       ABORT, NULL);
   #endif
 
-  if (list->singleton ())        //special case where
-                                 //we do need to
+  if (list->singleton()) {
+    // Special case where we do need to change the iterator.
     prev = next = list->last = NULL;
-  //      change the iterator
-  else {
+  } else {
     prev->next = next;           //remove from list
 
     if (current == list->last) {
       list->last = prev;
       ex_current_was_last = TRUE;
-    }
-    else
+    } else {
       ex_current_was_last = FALSE;
-
-    ex_current_was_cycle_pt = (current == cycle_pt) ? TRUE : FALSE;
-
+    }
   }
+  // Always set ex_current_was_cycle_pt so an add/forward will work in a loop.
+  ex_current_was_cycle_pt = (current == cycle_pt) ? TRUE : FALSE;
   extracted_data = current->data;
   delete(current);  //destroy CONS cell
   current = NULL;
@@ -640,7 +647,7 @@ inline void *CLIST_ITERATOR::extract() {
  **********************************************************************/
 
 inline void *CLIST_ITERATOR::move_to_first() {
-  #ifdef _DEBUG
+  #ifndef NDEBUG
   if (!this)
     NULL_OBJECT.error ("CLIST_ITERATOR::move_to_first", ABORT, NULL);
   if (!list)
@@ -650,7 +657,7 @@ inline void *CLIST_ITERATOR::move_to_first() {
   current = list->First ();
   prev = list->last;
   next = current != NULL ? current->next : NULL;
-  return current->data;
+  return current != NULL ? current->data : NULL;
 }
 
 
@@ -666,7 +673,7 @@ inline void *CLIST_ITERATOR::move_to_first() {
  **********************************************************************/
 
 inline void CLIST_ITERATOR::mark_cycle_pt() {
-  #ifdef _DEBUG
+  #ifndef NDEBUG
   if (!this)
     NULL_OBJECT.error ("CLIST_ITERATOR::mark_cycle_pt", ABORT, NULL);
   if (!list)
@@ -689,7 +696,7 @@ inline void CLIST_ITERATOR::mark_cycle_pt() {
  **********************************************************************/
 
 inline BOOL8 CLIST_ITERATOR::at_first() {
-  #ifdef _DEBUG
+  #ifndef NDEBUG
   if (!this)
     NULL_OBJECT.error ("CLIST_ITERATOR::at_first", ABORT, NULL);
   if (!list)
@@ -711,7 +718,7 @@ inline BOOL8 CLIST_ITERATOR::at_first() {
  **********************************************************************/
 
 inline BOOL8 CLIST_ITERATOR::at_last() {
-  #ifdef _DEBUG
+  #ifndef NDEBUG
   if (!this)
     NULL_OBJECT.error ("CLIST_ITERATOR::at_last", ABORT, NULL);
   if (!list)
@@ -733,7 +740,7 @@ inline BOOL8 CLIST_ITERATOR::at_last() {
  **********************************************************************/
 
 inline BOOL8 CLIST_ITERATOR::cycled_list() {
-  #ifdef _DEBUG
+  #ifndef NDEBUG
   if (!this)
     NULL_OBJECT.error ("CLIST_ITERATOR::cycled_list", ABORT, NULL);
   if (!list)
@@ -753,7 +760,7 @@ inline BOOL8 CLIST_ITERATOR::cycled_list() {
  **********************************************************************/
 
 inline inT32 CLIST_ITERATOR::length() {
-  #ifdef _DEBUG
+  #ifndef NDEBUG
   if (!this)
     NULL_OBJECT.error ("CLIST_ITERATOR::length", ABORT, NULL);
   if (!list)
@@ -775,7 +782,7 @@ inline void
 CLIST_ITERATOR::sort (           //sort elements
 int comparator (                 //comparison routine
 const void *, const void *)) {
-  #ifdef _DEBUG
+  #ifndef NDEBUG
   if (!this)
     NULL_OBJECT.error ("CLIST_ITERATOR::sort", ABORT, NULL);
   if (!list)
@@ -801,7 +808,7 @@ inline void CLIST_ITERATOR::add_to_end(  // element to add
                                        void *new_data) {
   CLIST_LINK *new_element;
 
-  #ifdef _DEBUG
+  #ifndef NDEBUG
   if (!this)
     NULL_OBJECT.error ("CLIST_ITERATOR::add_to_end", ABORT, NULL);
   if (!list)
